// #####################################################################
// #    MOUSE.CPP
// #    uses the mouse interrupt (0x33) with DOS/4GW.
// #    modified by Chrisi 7.2.1996
// #####################################################################
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <i86.h>
#include "mouse.h"

char mouse_handler_installed = 0;

extern "C" {
   struct callback_data;
   int lock_region (void *address, unsigned length);
   void _loadds far click_handler (int max, int mbx, int mcx, int mdx,
                                int msi, int mdi);
   void cbc_end (void);
           };



// #####################################################################
// # Data touched at mouse callback time -- they are in a structure to
// # simplify calculating the size of the region to lock.
// #####################################################################
struct callback_data
{
    int right_button;
    int mouse_event;
    unsigned short mouse_code;
    unsigned short mouse_bx;
    unsigned short mouse_cx;
    unsigned short mouse_dx;
    signed short mouse_si;
    signed short mouse_di;
} cbd = { 0 };


// #####################################################################
// # int lock_region (void *address, unsigned length)
// #####################################################################
int lock_region (void *address, unsigned length)
{
    union REGS regs;
    unsigned linear;

    // #####################################################################
    // # Thanks to DOS/4GW's zero-based flat memory model, converting
    // # a pointer of any type to a linear address is trivial.
    // #
    // #####################################################################
    linear = (unsigned) address;

    regs.w.ax = 0x600;                  // # DPMI Lock Linear Region
    regs.w.bx = (unsigned short) (linear >> 16); // # Linear address in BX:CX
    regs.w.cx = (unsigned short) (linear & 0xFFFF);
    regs.w.si = (unsigned short) (length >> 16); // # Length in SI:DI
    regs.w.di = (unsigned short) (length & 0xFFFF);
    int386 (0x31, &regs, &regs);
    return (! regs.w.cflag);            // # Return 0 if can't lock
}


// #####################################################################
// # void _loadds far click_handler ()
// #####################################################################
#pragma off (check_stack)
void _loadds far click_handler (int max, int mbx, int mcx, int mdx,
				int msi, int mdi)
{
#pragma aux click_handler parm [EAX] [EBX] [ECX] [EDX] [ESI] [EDI]
    cbd.mouse_event = 1;

    cbd.mouse_code = (unsigned short) max;
    cbd.mouse_bx = (unsigned short) mbx;
    cbd.mouse_cx = (unsigned short) mcx;
    cbd.mouse_dx = (unsigned short) mdx;
    cbd.mouse_si = (signed short) msi;
    cbd.mouse_di = (signed short) mdi;
    if (cbd.mouse_code & 8) cbd.right_button = 1;
}

void cbc_end (void) // # Dummy function so we can
{                   // # calculate size of code to lock
}                   // # (cbc_end - click_handler)
#pragma on (check_stack)


// #####################################################################
// # Install functions
// #####################################################################


void install_mouse_handler()
{
    struct SREGS sregs;
    union REGS inregs, outregs;
    int installed = 0;
   // unsigned char orig_mode = 0;
   // unsigned short far *ptr;
    void (far *function_ptr)();

    segread(&sregs);

    // # check for mouse driver

    inregs.w.ax = 0x21;
    int386 (0x33, &inregs, &outregs);
    installed = (outregs.w.ax == 0xffff);
  //  if( installed = (outregs.w.ax == 0xffff) )
  //      printf( "Mouse installed...\n" );
  //  else
  //      printf( "Mouse NOT installed...\n" );

    if (installed)
    {
        // #####################################################################
        // # lock callback code and data (essential under VMM!)
        // # note that click_handler, although it does a far return and
        // # is installed using a full 48-bit pointer, is really linked
        // # into the flat model code segment -- so we can use a regular
        // # (near) pointer in the lock_region() call.
        // #####################################################################
	if ( (! lock_region (&cbd, sizeof(cbd))) ||
	    (! lock_region ((void near *) click_handler,
	       (char *) cbc_end - (char near *) click_handler)))
	{
            // printf( "locks failed\n" );
	} else {

            // # install move + click watcher

	    inregs.w.ax = 0xC;
            inregs.w.cx = 0xff;
	    function_ptr = ( void(*)(void) ) click_handler;
	    inregs.x.edx = FP_OFF (function_ptr);
            sregs.es     = FP_SEG (function_ptr);
	    int386x (0x33, &inregs, &outregs, &sregs);

            mouse_handler_installed = 1;
            // Alright !
            atexit(deinstall_mouse_handler);
        }
    }

}

void deinstall_mouse_handler()
{
    union REGS inregs, outregs;

    if (mouse_handler_installed == 1)
     {
     // # check installation again (to clear watcher)

    inregs.w.ax = 0x21;
    int386 (0x33, &inregs, &outregs);
  //  if (outregs.w.ax == 0xffff)
  //      printf( "DONE : Mouse still installed...\n" );
  //  else
  //      printf( "DONE : Mouse NOT installed...\n" );

     }

}

// #####################################################################
// # access to mouse driver
// #####################################################################

void get_mouse_position(int & xpos, int & ypos)
{
    xpos = cbd.mouse_cx;
    ypos = cbd.mouse_dx;
}

int get_mouse_buttons()
{
    return(cbd.mouse_bx);
}

void set_mouse_position(int xpos, int ypos)
{
    union REGS r;
    struct SREGS s;
    s.ds = s.es = s.fs = s.gs = FP_SEG( &s );

    //cbd.mouse_cx = xpos;
    //cbd.mouse_dx = ypos;

    r.w.ax = 0x4;
    r.w.cx = xpos;
    r.w.dx = ypos;
    int386x (0x33, &r, &r, &s);

}

void set_mouse_region(int x1, int y1, int x2, int y2)
{
    union REGS r;
    struct SREGS s;
    s.ds = s.es = s.fs = s.gs = FP_SEG( &s );

    // Bereich in x-Richtung setzen
    r.w.ax = 0x7;
    r.w.cx = x1;
    r.w.dx = x2;
    int386x (0x33, &r, &r, &s);
    // Bereich in y-Richtung setzen
    r.w.ax = 0x8;
    r.w.cx = y1;
    r.w.dx = y2;
    int386x (0x33, &r, &r, &s);
}
